--[[

TradeSkillMaster Vendor addon
http://zotwee.info/

]]
local TSM = select(2, ...)
local GUI = TSM:NewModule("GUI", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendor")

local private = { data = {}, ignore = {} }
TSMAPI:RegisterForTracing(private, "TSM_Vendor.GUI_private")

function GUI:OnEnable()
	private.frame = private:CreateVendorFrame()
	TSMAPI:CreateEventBucket("BAG_UPDATE", function() private:UpdateST() end, 1)
	GUI:RegisterEvent("MERCHANT_SHOW", private.ShowFrame)
	GUI:RegisterEvent("MERCHANT_CLOSED", private.HideFrame)
	--private.ShowFrame()
end

function GUI:ShowFrame()
	private.hidden = nil
	private:UpdateST(true)
end

function private:ShowFrame()
  private.hidden = nil
  private:UpdateST(true)
end

function private:HideFrame()
  private.hidden = true
  private.frame:Hide()
end

function GUI:UpdateST(fromVendor)
	private:UpdateST(fromVendor)
end

function private:CreateVendorFrame()
  local frameDefaults = {
    x = 500,
    y = 450,
    width = 500,
    height = 400,
    scale = 1,
  }
  local frame = TSMAPI:CreateMovableFrame("TSMVendorFrame", frameDefaults)

	frame:SetFrameStrata("HIGH")
	TSMAPI.Design:SetFrameBackdropColor(frame)

	local title = TSMAPI.GUI:CreateLabel(frame)
	title:SetText("TSM_Vendor")
	title:SetPoint("TOPLEFT")
	title:SetPoint("TOPRIGHT")
	title:SetHeight(20)

	local line = TSMAPI.GUI:CreateVerticalLine(frame, 0)
	line:ClearAllPoints()
	line:SetPoint("TOPRIGHT", -25, -1)
	line:SetWidth(2)
	line:SetHeight(22)

	local closeBtn = TSMAPI.GUI:CreateButton(frame, 18)
	closeBtn:SetPoint("TOPRIGHT", -3, -3)
	closeBtn:SetWidth(19)
	closeBtn:SetHeight(19)
	closeBtn:SetText("X")
	closeBtn:SetScript("OnClick", function()
		if InCombatLockdown() then return end
		private.hidden = true
		frame:Hide()
	end)

	TSMAPI.GUI:CreateHorizontalLine(frame, -23)

	local stContainer = CreateFrame("Frame", nil, frame)
	stContainer:SetPoint("TOPLEFT", 0, -25)
	stContainer:SetPoint("BOTTOMRIGHT", 0, 30)
	TSMAPI.Design:SetFrameColor(stContainer)

	local stCols = {
		{
			name = L["Item"],
			width = 0.4,
		},
		{
      name = L["Price"],
      width = 0.23,
      align = "RIGHT",
    },
    {
      name = L["Qty"],
      width = 0.05,
      align = "RIGHT",
    },
    {
      width = 0.02,
    },
		{
			name = L["Operation"],
			width = 0.32,
		},
	}
	local handlers = {
		OnClick = function(_, data, self, button)
			if not data then return end
			if button == "RightButton" then
				if IsShiftKeyDown() then
					TSM.db.global.ignore[data.itemString] = true
					TSM:Printf(L["Ignoring %s permanently. You can undo this in the TSM Vendor options."], data.link)
					TSM.Options:UpdateIgnoreST()
				else
					private.ignore[data.itemString] = true
					TSM:Printf(L["Ignoring %s this session (until your UI is reloaded)."], data.link)
				end
				private:UpdateST()
			end
		end,
		OnEnter = function(_, data, self)
			if not data then return end
			local color = TSMAPI.Design:GetInlineColor("link")
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:AddLine(data.link)
			GameTooltip:AddLine(format(L["%sRight-Click|r to ignore this item for this session. %sShift + Right-Click|r to ignore permanently. You can remove items from permanent ignore in the Vendor options."], color, color), 1, 1, 1, 1, true)
			GameTooltip:Show()
		end,
		OnLeave = function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
		end
	}
	local st = TSMAPI:CreateScrollingTable(stContainer, stCols, handlers, 12)
	st:SetData({})
	st:DisableSelection(true)
	frame.st = st

	local sellNextBtn = TSMAPI.GUI:CreateButton(frame, 14, "TSMSellNextButton", true)
	sellNextBtn:SetPoint("BOTTOMLEFT", 3, 3)
	sellNextBtn:SetWidth(150)
	sellNextBtn:SetHeight(20)
	sellNextBtn:SetText(L["Sell Next"])
	sellNextBtn:SetScript("PreClick", function()
		if sellNextBtn:IsVisible() and #private.data > 0 then
			local data = private.data[1]
			private.tempData = data
			TSM:Printf(L["Selling %s. Reason: %s."], data.link, data.cols[5].value)
      UseContainerItem(data.bag, data.slot)
      if data.vendorSell and data.vendorSell > 0 then
        TSM:Print(format(L["Sold %d item(s) for %s."], data.quantity, TSMAPI:FormatTextMoneyIcon(data.vendorSell, "|cffffffff", true)))
      end
		end
	end)
	frame.sellNextBtn = sellNextBtn

  local sellAllBtn = TSMAPI.GUI:CreateButton(frame, 14, "TSMSellAllButton", true)
  sellAllBtn:SetPoint("BOTTOMLEFT", 160, 3)
  sellAllBtn:SetWidth(150)
  sellAllBtn:SetHeight(20)
  sellAllBtn:SetText(L["Sell All"])
  sellAllBtn:SetScript("PreClick", function()
    if sellAllBtn:IsVisible() and #private.data > 0 then
      local soldQty = 0
      local soldTotal = 0
      for i = 1 , #private.data, 1 do
        local data = private.data[i]
        private.tempData = data
        TSM:Printf(L["Selling %s. Reason: %s."], data.link, data.cols[5].value)
        UseContainerItem(data.bag, data.slot)
        soldQty = soldQty + data.quantity
        soldTotal = soldTotal + data.vendorSell
      end
      if soldTotal and soldTotal > 0 then
        TSM:Print(format(L["Sold %d items for %s."], soldQty, TSMAPI:FormatTextMoneyIcon(soldTotal, "|cffffffff", true)))
      end
    end
  end)
  frame.sellAllBtn = sellAllBtn

	return frame
end

function private:SellItem(bag, slot, reason)
  local link = GetContainerItemLink(bag, slot)
  TSM:Printf(L["Selling %s. Reason: %s."], link, reason)
  UseContainerItem(bag, slot)
end

function private:UpdateST(fromVendor)
	if private.hidden then return end
	if InCombatLockdown() then return end

	local stData = {}
	local soldQty = 0
	local soldTotal = 0
	for bag, slot, itemString, quantity in TSMAPI:GetBagIterator(nil, true) do
		if not private.ignore[itemString] and not TSM.db.global.ignore[itemString] then
				local link = GetContainerItemLink(bag, slot)
				local itemName, _, itemQuality, _, _, itemType, itemSubType = GetItemInfo(link)
        local vendorSell = TSM:GetPrice("VendorSell", itemString) or 0
        local toSell = false
        local reason = TSM:QualityText(0)
        local WEAPON, ARMOR = GetAuctionItemClasses()
        if vendorSell and vendorSell > 0 then
  				if fromVendor and itemQuality and itemQuality == 0 and TSM.db.global.autoSellGray then
  				  private:SellItem(bag,slot,L["Gray"])
  				  soldTotal = soldTotal + vendorSell * quantity
  				  soldQty = soldQty + quantity
  				  toSell = false
  				else
    				if not toSell and TSM.db.global.sellSoulBound and TSMAPI:IsSoulbound(bag, slot) and itemQuality and itemQuality <= TSM.db.global.maxSoulBoundQuality and (itemType == ARMOR or itemType == WEAPON) then
    				  toSell = true
    				  reason = L["SoulBound"]
    				end
  
            local ops = TSMAPI:GetItemOperation(itemString, "Vendor")
            if not toSell and ops and #ops > 0 then
              for _, operationName in pairs(ops) do
                TSMAPI:UpdateOperation("Vendor", operationName)
                maxSellQuality = TSM.operations[operationName].maxSellQuality
                if itemQuality and itemQuality <= maxSellQuality then
                  toSell = true
                  reason = operationName..": "..L["Quality"]
                end
                if not toSell then
                  local cancelledNum, expiredNum, totalFailed = TSMAPI:ModuleAPI("Accounting","getAuctionStatsSinceLastSale",itemString)
                  if expiredNum >= TSM.operations[operationName].numberExpired and itemQuality <= TSM.operations[operationName].maxExpiredQuality then
                    toSell = true
                    reason = operationName..": "..expiredNum.." "..L["Expired Auctions"]
                  end
                end
                
              end
            end
          end
                    
          if fromVendor and toSell and TSM.db.global.autoSell then
            private:SellItem(bag,slot,reason)
            soldTotal = soldTotal + vendorSell * quantity
            soldQty = soldQty + quantity
            toSell = false
          end
  
  				if toSell then
  					local row = {
  						cols = {
  							{
  								value = link
  							},
  							{
  							  value = TSMAPI:FormatTextMoneyIcon(vendorSell, "|cffffffff", true)
  							},
  							{
                  value = quantity
                },
                {},
  							{
  								value = reason
  							},
  						},
  						itemString = itemString,
  						link = link,
  						quantity = quantity,
  						bag = bag,
  						slot = slot,
  						vendorSell = vendorSell,
  					}
  					tinsert(stData, row)
  				end
  		  end
		end
	end
	
	if soldTotal and soldTotal > 0 then
	  TSM:Print(format(L["Sold %d items for %s."], soldQty, TSMAPI:FormatTextMoneyIcon(soldTotal, "|cffffffff", true)))
	end

	if #stData == 0 then
		if fromVendor and (not soldTotal or soldTotal == 0) then
			TSM:Print(L["Nothing to sell in your bags."])
		end
		private.frame.sellNextBtn:Disable()
		private.frame.sellAllBtn:Disable()
		private.frame:Hide()
	elseif fromVendor and not private.frame:IsVisible() then
    private.frame.sellNextBtn:Enable()
    private.frame.sellAllBtn:Enable()
		private.frame:Show()
	end
	private.data = CopyTable(stData)
	private.frame.st:SetData(stData)
end
